//==============================================================================
//	
//	Copyright (c) 2011-
//	Authors:
//	* Christian Krause <christian.krause@hpi.uni-potsdam.de> (Hasso Plattner Institute)
//	
//------------------------------------------------------------------------------
//	
//	This file is part of PRISM.
//	
//	PRISM is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	PRISM is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with PRISM; if not, write to the Free Software Foundation,
//	Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//==============================================================================

package explicit;

import java.util.HashMap;
import java.util.Map;

import prism.PrismUtils;

public class IntervalDistribution extends Distribution {

	protected Map<Integer,Double> upperBounds;

	/**
	 * Default constructor.
	 */
	public IntervalDistribution()
	{
		super();
	}
	
	/*
	 * (non-Javadoc)
	 * @see explicit.Distribution#clear()
	 */
	@Override
	public void clear()
	{	
		super.clear();
		upperBounds = new HashMap<Integer,Double>();
	}

	/**
	 * Get the lower bounds of this interval distribution.
	 * @return Lower bounds.
	 */
	protected Map<Integer,Double> getLowerBounds()
	{
		return map;
	}

	/**
	 * Get the upper bounds of this interval distribution.
	 * @return Upper bounds.
	 */
	protected Map<Integer,Double> getUpperBounds()
	{
		return upperBounds;
	}
	
	@Override
	public boolean add(int j, double prob)
	{
		throw new UnsupportedOperationException();
	}
	
	public boolean add(int j, double lower, double upper)
	{
		Double l = getLowerBounds().get(j);
		Double u = getUpperBounds().get(j);
		if (l==null || u==null) {
			getLowerBounds().put(j,lower);
			getUpperBounds().put(j,upper);
			return false;
		} else {
			set(j,l+lower,u+upper);
			return true;
		}
	}

	@Override
	public void set(int j, double prob)
	{
		throw new UnsupportedOperationException();
	}

	/**
	 * Set the probability interval for state 'j' to 'lower~upper'.
	 */
	public void set(int j, double lower, double upper)
	{
		if (upper==0.0) {
			getLowerBounds().remove(j);
			getUpperBounds().remove(j);
		} else {
			getLowerBounds().put(j,lower);
			getUpperBounds().put(j,upper);			
		}
	}

	public double get(int j)
	{
		throw new UnsupportedOperationException();
	}
	
	public double getLower(int j)
	{
		Double d = (Double) getLowerBounds().get(j);
		return d==null ? 0.0 : d.doubleValue();
	}

	public double getUpper(int j)
	{
		Double d = (Double) getUpperBounds().get(j);
		return d==null ? 0.0 : d.doubleValue();
	}

	@Override
	public double sum()
	{
		throw new UnsupportedOperationException();
	}

	public double sumLower()
	{
		double sumLower = 0.0;
		Map<Integer,Double> lower = getLowerBounds();
		for (Integer state : states()) {
			sumLower += lower.get(state);
		}
		return sumLower;
	}

	public double sumUpper()
	{
		double sumUpper = 0.0;
		for (Integer state : states()) {
			sumUpper += upperBounds.get(state);
		}
		return sumUpper;
	}
	
	public double sumAllBut(int j)
	{
		throw new UnsupportedOperationException();
	}
	
	@Override
	public IntervalDistribution map(int map[])
	{
		IntervalDistribution distrNew = new IntervalDistribution();
		Map<Integer,Double> lowerBounds = getLowerBounds();
		for (Integer state : states()) {
			distrNew.add(map[state], lowerBounds.get(state), upperBounds.get(state));
		}
		return distrNew;
	}
	
	@Override
	public boolean equals(Object o)
	{
		if (!(o instanceof IntervalDistribution)) {
			return false;
		}		
		IntervalDistribution d = (IntervalDistribution) o;
		if (d.size()!=size()) {
			return false;
		}
		for (Integer state : states()) {
			Double l1 = getLower(state);
			Double u1 = getUpper(state);
			Double l2 = d.getLower(state);
			Double u2 = d.getUpper(state);
			if (l2==null || u2==null || 
				!PrismUtils.doublesAreClose(l1, l2, 1e-12, false) ||
				!PrismUtils.doublesAreClose(u1, u2, 1e-12, false)) {
				return false;
			}
		}
		return true;
	}

	@Override
	public int hashCode()
	{
		// Simple hash code
		return map.size();
	}

	@Override
	public String toString()
	{
		String result = "";
		for (Integer state : states()) {
			if (result.length()>0) result = result + ",";
			result = result + state + "=" + getLower(state) + "~" + getUpper(state);
		}
		return "[" + result + "]";
	}

}
