//==============================================================================
//	
//	Copyright (c) 2011-
//	Authors:
//	* Christian Krause <christian.krause@hpi.uni-potsdam.de> (Hasso Plattner Institute)
//	
//------------------------------------------------------------------------------
//	
//	This file is part of PRISM.
//	
//	PRISM is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	PRISM is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with PRISM; if not, write to the Free Software Foundation,
//	Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//==============================================================================

package ipta;

import java.util.BitSet;

import parser.ast.ModulesFile;
import parser.ast.PropertiesFile;
import prism.Prism;
import prism.PrismException;
import prism.PrismSettings;
import pta.PTAModelChecker;

public class IPTAModelChecker extends PTAModelChecker {

	/**
	 * Default constructor.
	 * @param prism Prism core.
	 * @param modulesFile Modules file.
	 * @param propertiesFile Properties file.
	 */
	public IPTAModelChecker(Prism prism, ModulesFile modulesFile, PropertiesFile propertiesFile) throws PrismException
	{
		super(prism, modulesFile, propertiesFile);
	}

	/*
	 * (non-Javadoc)
	 * @see pta.PTAModelChecker#buildPTA()
	 */
	@Override
	protected IPTA buildPTA() throws PrismException {
		mainLog.println("\nBuilding IPTA...");
		Modules2IPTA m2ipta = new Modules2IPTA(prism, modulesFile);
		IPTA ipta = m2ipta.translate();
		mainLog.println("\nIPTA: " + ipta.infoString());
		return ipta;
	}

	/*
	 * (non-Javadoc)
	 * @see pta.PTAModelChecker#computeProbabilisticReachability(java.util.BitSet, boolean)
	 */
	@Override
	protected double computeProbabilisticReachability(BitSet targetLocs, boolean min) throws PrismException
	{
		// Determine which method to use for computation
		String ptaMethod = prism.getSettings().getString(PrismSettings.PRISM_PTA_METHOD);

		// Do probability computation through abstraction/refinement/stochastic games  
		if (ptaMethod.equals("Stochastic games")) {
			IPTAAbstractRefine iptaAR;
			iptaAR = new IPTAAbstractRefine();
			String arOptions = prism.getSettings().getString(PrismSettings.PRISM_AR_OPTIONS);
			iptaAR.setLog(mainLog);
			iptaAR.parseOptions(arOptions.split(","));
			return iptaAR.forwardsReachAbstractRefine(pta, targetLocs, null, min);
		}
		// Do probability computation by first constructing a bisimulation  
		else if (ptaMethod.equals("Bisimulation minimisation")) {
			throw new PrismException("Not yet supported");
		}
		else {
			throw new PrismException("Unknown IPTA solution method");
		}
	}

}
