//==============================================================================
//	
//	Copyright (c) 2011-
//	Authors:
//	* Christian Krause <christian.krause@hpi.uni-potsdam.de> (Hasso Plattner Institute)
//	
//------------------------------------------------------------------------------
//	
//	This file is part of PRISM.
//	
//	PRISM is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	PRISM is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with PRISM; if not, write to the Free Software Foundation,
//	Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//==============================================================================

package ipta;

import pta.Edge;
import pta.PTA;
import pta.PTAParallel;
import pta.Transition;

/**
 * Compose two IPTA in parallel.
 */
public class IPTAParallel extends PTAParallel {
	
	/*
	 * (non-Javadoc)
	 * @see pta.PTAParallel#createPTA()
	 */
	@Override
	protected PTA createParallel()
	{
		return new IPTA();
	}

	/*
	 * (non-Javadoc)
	 * @see pta.PTAParallel#compose(pta.PTA, pta.PTA)
	 */
	@Override
	public PTA compose(PTA pta1, PTA pta2)
	{
		if (!(pta1 instanceof IPTA) || !(pta2 instanceof IPTA)) {
			throw new IllegalArgumentException();
		} else {
			return super.compose(pta1, pta2);
		}
	}
	
	/*
	 * Add an asynchronous edge to a transition.
	 * This is a helper method that can be overridden.
	 */
	protected Edge addAsynchronousEdge(Transition transition, Edge edge, int dest) 
	{
		double lower = ((IntervalEdge) edge).getLowerBound();
		double upper = ((IntervalEdge) edge).getUpperBound();
		return ((IntervalTransition) transition).addEdge(lower, upper, dest);
	}
	
	/*
	 * Add a synchronous edge to a transition.
	 * This is a helper method that can be overridden.
	 */
	protected Edge addSynchronousEdge(Transition transition, Edge edge1, Edge edge2, int dest) 
	{
		double lower1 = ((IntervalEdge) edge1).getLowerBound();
		double lower2 = ((IntervalEdge) edge2).getLowerBound();
		double upper1 = ((IntervalEdge) edge1).getUpperBound();
		double upper2 = ((IntervalEdge) edge2).getUpperBound();
		return ((IntervalTransition) transition).addEdge(lower1*lower2, upper1*upper2, dest);
	}
}
