//==============================================================================
//	
//	Copyright (c) 2011-
//	Authors:
//	* Christian Krause <christian.krause@hpi.uni-potsdam.de> (Hasso Plattner Institute)
//	
//------------------------------------------------------------------------------
//	
//	This file is part of PRISM.
//	
//	PRISM is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	PRISM is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with PRISM; if not, write to the Free Software Foundation,
//	Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//==============================================================================

package ipta;

import java.io.IOException;
import java.io.Writer;
import java.util.Map;

import prism.PrismException;
import pta.Constraint;
import pta.Edge;
import pta.Transition;

public class IntervalEdge extends Edge {
	
	protected double upper;
	
	public IntervalEdge(IntervalTransition parent, double lower, double upper, int dest)
	{
		super(parent, lower, dest);
		this.upper = upper;
	}
	
	/**
	 * Copy this edge.
	 */
	@Override
	public IntervalEdge copy()
	{
		IntervalEdge copy = new IntervalEdge((IntervalTransition) parent, getLowerBound(), upper, dest);
		for (Map.Entry<Integer, Integer> e : resets.entrySet()) {
			copy.addReset(e.getKey(), e.getValue());
		}
		return copy;
	}

	public double getLowerBound() 
	{
		return getProbability();
	}

	public void setLowerBound(double lower) 
	{
		setProb(lower);
	}
	
	public double getUpperBound() 
	{
		return upper;
	}

	public void setUpperBound(double upper) 
	{
		this.upper = upper;
	}
	
	@Override
	public IntervalTransition getParent()
	{
		return (IntervalTransition) super.getParent();
	}
	
	@Override
	public void setParent(Transition parent)
	{
		if (!(parent instanceof IntervalTransition)) {
			throw new RuntimeException("Parent must be an interval transition");
		} else {
			super.setParent(parent);
		}
	}
	
	@Override
	public void check() throws PrismException
	{
		if (getLowerBound()<0 || getLowerBound()>1 || 
			getUpperBound()<0 || getUpperBound()>1) {
			throw new PrismException("Lower and upper bounds must be between 0 and 1");
		}
		if (getLowerBound()>getUpperBound()) {
			throw new PrismException("Lower bound (" + getLowerBound() + 
					") must be smaller then upper bound (" + getUpperBound() + ")");
		}
	}
	
	@Override
	public String toString()
	{
		String result = super.toString();
		if (getLowerBound()!=getUpperBound()) {
			result = result.replaceFirst("" + getProbability(), 
					getLowerBound() + "~" + getUpperBound());
		}
		return result;
	}

	@Override
	public void writeToDesFile(Writer out, String actionPrefix, Iterable<Constraint> guard) throws PrismException, IOException
	{
		throw new UnsupportedOperationException();
	}
}